@@ -2,7 +2,7 @@
  * access.c: Real rtsp input
  *****************************************************************************
  * Copyright (C) 2005 VideoLAN
- * $Id: a1492a69af0be2192e3a7464062b6bdb73c663bc $
+ * $Id$
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -31,7 +31,7 @@
 #include <vlc_common.h>
 #include <vlc_plugin.h>
 #include <vlc_access.h>
-#include <vlc_interface.h>
+#include <vlc_dialog.h>
 
 #include <vlc_network.h>
 #include "rtsp.h"
@@ -48,19 +48,20 @@ static void Close( vlc_object_t * );
     "Caching value for RTSP streams. This " \
     "value should be set in milliseconds." )
 
-vlc_module_begin();
-    set_description( N_("Real RTSP") );
-    set_shortname( N_("Real RTSP") );
-    set_category( CAT_INPUT );
-    set_subcategory( SUBCAT_INPUT_ACCESS );
+vlc_module_begin ()
+    set_description( N_("Real RTSP") )
+    set_shortname( N_("Real RTSP") )
+    set_category( CAT_INPUT )
+    set_subcategory( SUBCAT_INPUT_ACCESS )
     add_integer( "realrtsp-caching", 3000, NULL,
-                 CACHING_TEXT, CACHING_LONGTEXT, true );
-    set_capability( "access", 10 );
-    set_callbacks( Open, Close );
-    add_shortcut( "realrtsp" );
-    add_shortcut( "rtsp" );
-    add_shortcut( "pnm" );
-vlc_module_end();
+                 CACHING_TEXT, CACHING_LONGTEXT, true )
+        change_safe()
+    set_capability( "access", 10 )
+    set_callbacks( Open, Close )
+    add_shortcut( "realrtsp" )
+    add_shortcut( "rtsp" )
+    add_shortcut( "pnm" )
+vlc_module_end ()
 
 
 /*****************************************************************************
@@ -95,7 +96,7 @@ static int RtspConnect( void *p_userdata, char *psz_server, int i_port )
     if( p_sys->fd < 0 )
     {
         msg_Err( p_access, "cannot connect to %s:%d", psz_server, i_port );
-        intf_UserFatal( p_access, false, _("Connection failed"),
+        dialog_Fatal( p_access, _("Connection failed"),
                         _("VLC could not connect to \"%s:%d\"."), psz_server, i_port );
         return VLC_EGENERIC;
     }
@@ -155,7 +156,7 @@ static int Open( vlc_object_t *p_this )
 {
     access_t *p_access = (access_t *)p_this;
     access_sys_t *p_sys;
-    char *psz_server = 0;
+    char* psz_server = NULL;
     int i_result;
 
     if( !p_access->psz_access || (
@@ -177,9 +178,16 @@ static int Open( vlc_object_t *p_this )
     p_access->info.i_title = 0;
     p_access->info.i_seekpoint = 0;
     p_access->p_sys = p_sys = malloc( sizeof( access_sys_t ) );
+    if( !p_sys )
+        return VLC_ENOMEM;
     p_sys->p_rtsp = malloc( sizeof( rtsp_client_t) );
+    if( !p_sys->p_rtsp )
+    {
+        free( p_sys );
+        return VLC_ENOMEM;
+    }
 
-    p_sys->p_header = 0;
+    p_sys->p_header = NULL;
     p_sys->p_rtsp->p_userdata = p_access;
     p_sys->p_rtsp->pf_connect = RtspConnect;
     p_sys->p_rtsp->pf_disconnect = RtspDisconnect;
@@ -192,7 +200,7 @@ static int Open( vlc_object_t *p_this )
     {
         msg_Dbg( p_access, "could not connect to: %s", p_access->psz_path );
         free( p_sys->p_rtsp );
-        p_sys->p_rtsp = 0;
+        p_sys->p_rtsp = NULL;
         goto error;
     }
 
@@ -229,7 +237,7 @@ static int Open( vlc_object_t *p_this )
 
 
             msg_Err( p_access, "rtsp session can not be established" );
-            intf_UserFatal( p_access, false, _("Session failed"),
+            dialog_Fatal( p_access, _("Session failed"),
                     _("The requested RTSP session could not be established.") );
             goto error;
         }
@@ -284,12 +292,12 @@ static block_t *BlockRead( access_t *p_access )
     if( p_sys->p_header )
     {
         p_block = p_sys->p_header;
-        p_sys->p_header = 0;
+        p_sys->p_header = NULL;
         return p_block;
     }
 
     i_size = real_get_rdt_chunk_header( p_access->p_sys->p_rtsp, &pheader );
-    if( i_size <= 0 ) return 0;
+    if( i_size <= 0 ) return NULL;
 
     p_block = block_New( p_access, i_size );
     p_block->i_buffer = real_get_rdt_chunk( p_access->p_sys->p_rtsp, &pheader,
@@ -311,9 +319,8 @@ static int Seek( access_t *p_access, int64_t i_pos )
  *****************************************************************************/
 static int Control( access_t *p_access, int i_query, va_list args )
 {
-    bool   *pb_bool;
-    int          *pi_int;
-    int64_t      *pi_64;
+    bool        *pb_bool;
+    int64_t     *pi_64;
 
     switch( i_query )
     {
@@ -334,12 +341,6 @@ static int Control( access_t *p_access, int i_query, va_list args )
             *pb_bool = true;//p_sys->b_pace_control;
             break;
 
-        /* */
-        case ACCESS_GET_MTU:
-            pi_int = (int*)va_arg( args, int * );
-            *pi_int = 0;
-            break;
-
         case ACCESS_GET_PTS_DELAY:
             pi_64 = (int64_t*)va_arg( args, int64_t * );
             *pi_64 = var_GetInteger( p_access, "realrtsp-caching" ) * 1000;
