@@ -1,8 +1,8 @@
 /*****************************************************************************
  * equalizer_presets.h:
  *****************************************************************************
- * Copyright (C) 2004 VideoLAN
- * $Id: equalizer_presets.h 10101 2005-03-02 16:47:31Z robux4 $
+ * Copyright (C) 2004 the VideoLAN team
+ * $Id$
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -30,12 +30,13 @@
 
 #define EQZ_BANDS_MAX 10
 
-static char *preset_list[] = {
+#define NB_PRESETS 18
+static const char *const preset_list[] = {
     "flat", "classical", "club", "dance", "fullbass", "fullbasstreble",
     "fulltreble", "headphones","largehall", "live", "party", "pop", "reggae",
     "rock", "ska", "soft", "softrock", "techno"
 };
-static char *preset_list_text[] = {
+static const char *const preset_list_text[] = {
     N_("Flat"), N_("Classical"), N_("Club"), N_("Dance"), N_("Full bass"),
     N_("Full bass and treble"), N_("Full treble"), N_("Headphones"),
     N_("Large Hall"), N_("Live"), N_("Party"), N_("Pop"), N_("Reggae"),
@@ -44,7 +45,7 @@ static char *preset_list_text[] = {
 
 typedef struct
 {
-    char *psz_name;
+    const char *psz_name;
     int  i_band;
     float f_preamp;
     float f_amp[EQZ_BANDS_MAX];
