@@ -2,7 +2,7 @@
  * encoder.c: video and audio encoder using the ffmpeg library
  *****************************************************************************
  * Copyright (C) 1999-2004 the VideoLAN team
- * $Id: 1eb037f278130656c71d4e289bfca9d95ed55a31 $
+ * $Id: 4aefaec0c6acc63c8be3f22effaced1b2810c28f $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -237,6 +237,12 @@ int OpenEncoder( vlc_object_t *p_this )
         return VLC_EGENERIC;
     }
 
+    if( p_enc->fmt_out.i_cat == SPU_ES )
+    {
+        /* We don't support subtitle encoding */
+        return VLC_EGENERIC;
+    }
+
     /* Initialization must be done before avcodec_find_encoder() */
     InitLibavcodec( p_this );
 
@@ -700,11 +706,11 @@ int OpenEncoder( vlc_object_t *p_this )
         if( p_enc->fmt_out.p_extra )
         {
             uint8_t *p = p_enc->fmt_out.p_extra;
-            p[0] = 0x66;
-            p[1] = 0x4C;
-            p[2] = 0x61;
-            p[3] = 0x43;
-            p[4] = 0x00;
+            p[0] = 0x66;    /* f */
+            p[1] = 0x4C;    /* L */
+            p[2] = 0x61;    /* a */
+            p[3] = 0x43;    /* C */
+            p[4] = 0x80;    /* streaminfo block, last block before audio */
             p[5] = ( p_context->extradata_size >> 16 ) & 0xff;
             p[6] = ( p_context->extradata_size >>  8 ) & 0xff;
             p[7] = ( p_context->extradata_size       ) & 0xff;
