@@ -3,9 +3,9 @@
  *****************************************************************************
  * Copyright (C) 2003-2004 Commonwealth Scientific and Industrial Research
  *                         Organisation (CSIRO) Australia
- * Copyright (C) 2004 the VideoLAN team
+ * Copyright (C) 2004-2008 the VideoLAN team
  *
- * $Id: 73c6833d27de3aa64d63556eb5e94968724a6b74 $
+ * $Id$
  *
  * Authors: Andre Pang <Andre.Pang@csiro.au>
  *
@@ -31,13 +31,7 @@
 
 #include "xurl.h"
 
-static char *streallocat( char *psz_string, char *psz_to_append );
-
-#ifndef HAVE_STRDUP
-static char *xurl_strdup( const char *psz_string );
-#else
-#define xurl_strdup strdup
-#endif
+static char *streallocat( char *psz_string, const char *psz_to_append );
 
 char        *XURL_FindQuery             ( char *psz_url );
 static char *XURL_FindHostname          ( char *psz_url );
@@ -51,8 +45,6 @@ char *XURL_Join( char *psz_url1, char *psz_url2 )
         return XURL_Concat( psz_url1, psz_url2 );
     else
         return XURL_Concat( psz_url2, psz_url1 );
-
-    return NULL;
 }
 
 /* TODO: replace XURL_Concat's rel/absolute calculation with the one
@@ -63,7 +55,7 @@ char *XURL_Concat( char *psz_url, char *psz_append )
 {
     char *psz_return_value = NULL;
 
-    if( XURL_IsAbsolute( psz_append ) == XURL_TRUE )
+    if( XURL_IsAbsolute( psz_append ) )
         return strdup( psz_append );
 
     if( XURL_IsAbsolute( psz_url ) )
@@ -96,7 +88,7 @@ char *XURL_Concat( char *psz_url, char *psz_append )
     else
     {
         /* not an absolute URL */
-        if( XURL_HasAbsolutePath( psz_append ) == XURL_FALSE )
+        if( XURL_HasAbsolutePath( psz_append ) == false )
         {
             char *psz_new_url = XURL_GetHead( psz_url );
 
@@ -106,7 +98,7 @@ char *XURL_Concat( char *psz_url, char *psz_append )
         else
         {
             /* URL to append has an absolute path -- just use that instead */
-            psz_return_value = xurl_strdup( psz_append );
+            psz_return_value = strdup( psz_append );
         }
     }
 
@@ -114,31 +106,31 @@ char *XURL_Concat( char *psz_url, char *psz_append )
 }
 
 
-XURL_Bool XURL_IsAbsolute( char *psz_url )
+bool XURL_IsAbsolute( char *psz_url )
 {
     if( XURL_FindHostname( psz_url ) == NULL )
     {
 #ifdef XURL_DEBUG
         fprintf( stderr, "XURL_IsAbsolute(%s) returning false\n", psz_url );
 #endif
-        return XURL_FALSE;
+        return false;
     }
     else
     {
 #ifdef XURL_DEBUG
         fprintf( stderr, "XURL_IsAbsolute(%s) returning true\n", psz_url );
 #endif
-        return XURL_TRUE;
+        return true;
     }
 }
 
 
-XURL_Bool XURL_HasFragment( char *psz_url )
+bool XURL_HasFragment( char *psz_url )
 {
     if( XURL_FindFragment( psz_url ) == NULL )
-        return XURL_FALSE;
+        return false;
     else
-        return XURL_TRUE;
+        return true;
 }
 
 
@@ -150,7 +142,8 @@ char *XURL_FindHostname( char *psz_url )
     if( psz_scheme_separator != NULL)
     {
         char *psz_hostname = psz_scheme_separator + strlen( "://" );
-        if( *psz_hostname != '\0') psz_return_value = psz_hostname;
+        if( *psz_hostname != '\0')
+            psz_return_value = psz_hostname;
 
 #ifdef XURL_DEBUG
         fprintf( stderr, "XURL_FindHostname(%s): returning \"%s\"\n",
@@ -162,16 +155,16 @@ char *XURL_FindHostname( char *psz_url )
 }
 
 
-XURL_Bool XURL_HasAbsolutePath( char *psz_url )
+bool XURL_HasAbsolutePath( char *psz_url )
 {
 #ifdef XURL_WIN32_PATHING
     if( psz_url[0] == '/' || psz_url[0] == '\\' )
 #else
     if( psz_url[0] == '/' )
 #endif
-        return XURL_TRUE;
+        return true;
     else
-        return XURL_FALSE;
+        return false;
 }
 
 
@@ -201,8 +194,9 @@ char *XURL_GetHostname( char *psz_url )
         }
 
         /* Copy hostname to a new string */
-        psz_new_hostname = xurl_malloc( i_hostname_length );
-        if (psz_new_hostname == NULL) return NULL;
+        psz_new_hostname = malloc( i_hostname_length );
+        if( psz_new_hostname == NULL )
+            return NULL;
         strncpy( psz_new_hostname, psz_hostname, i_hostname_length );
 
 #ifdef XURL_DEBUG
@@ -223,38 +217,31 @@ char *XURL_GetHostname( char *psz_url )
 
 char *XURL_GetSchemeAndHostname( char *psz_url )
 {
-    char *psz_scheme, *psz_hostname, *psz_scheme_and_hostname;
+    char *psz_scheme = NULL,
+         *psz_hostname = NULL,
+         *psz_scheme_and_hostname = NULL;
 
     psz_scheme = XURL_GetScheme( psz_url );
-    if( psz_scheme == NULL ) return NULL;
-
     psz_hostname = XURL_GetHostname( psz_url );
-    if( psz_hostname == NULL ) return NULL;
+    if( psz_hostname && psz_scheme )
+    {
+        if( asprintf( &psz_scheme_and_hostname, "%s://%s", psz_scheme, psz_hostname ) == -1)
+            psz_scheme_and_hostname = NULL;
+    }
 
-    /* malloc +1 for the terminating '\0' */
-    psz_scheme_and_hostname = xurl_malloc(
-            strlen( psz_scheme ) + strlen( "://" ) +
-            strlen( psz_hostname ) + 1);
-    if( psz_scheme_and_hostname == NULL ) return NULL;
-    (void) strcpy( psz_scheme_and_hostname, psz_scheme );
-    (void) strcat( psz_scheme_and_hostname, "://" );
-    (void) strcat( psz_scheme_and_hostname, psz_hostname );
-
-    if (psz_scheme_and_hostname == NULL ) return NULL;
+    free( psz_hostname );
+    free( psz_scheme );
     return psz_scheme_and_hostname;
 }
 
-static
-char *XURL_FindFragment( char *psz_url )
+static char *XURL_FindFragment( char *psz_url )
 {
     char *pc_hash = NULL;
     char *pc_return_value = NULL;
  
     pc_hash = strchr( psz_url, '#' );
     if( pc_hash != NULL )
-    {
         pc_return_value = pc_hash;
-    }
 
     return pc_return_value;
 }
@@ -266,9 +253,7 @@ char *XURL_FindQuery( char *psz_url )
  
     pc_question_mark = strchr( psz_url, '?' );
     if( pc_question_mark != NULL )
-    {
         pc_return_value = pc_question_mark;
-    }
 
     return pc_return_value;
 }
@@ -280,7 +265,8 @@ char *XURL_GetScheme( char *psz_url )
     size_t i_scheme_length;
     char *new_scheme;
 
-    if( XURL_IsAbsolute( psz_url ) == XURL_FALSE ) return strdup( "file" );
+    if( XURL_IsAbsolute( psz_url ) == false )
+        return strdup( "file" );
 
     /* this strchr will always succeed since we have an absolute URL, and thus
      * a scheme */
@@ -288,76 +274,48 @@ char *XURL_GetScheme( char *psz_url )
 
     i_scheme_length = psz_colon - psz_url;
 
-    new_scheme = xurl_malloc( i_scheme_length );
-    if( new_scheme == NULL ) return NULL;
+    new_scheme = malloc( i_scheme_length );
+    if( new_scheme == NULL )
+        return NULL;
 
     strncpy( new_scheme, psz_url, i_scheme_length );
-
     return new_scheme;
 }
 
 
-XURL_Bool XURL_IsFileURL( char *psz_url )
+bool XURL_IsFileURL( char *psz_url )
 {
-    XURL_Bool b_return_value;
+    bool b_return_value;
     char *psz_scheme = XURL_GetScheme( psz_url );
 
     if( strcasecmp( psz_scheme, "file" ) == 0 )
-        b_return_value = XURL_TRUE;
+        b_return_value = true;
     else
-        b_return_value = XURL_FALSE;
+        b_return_value = false;
 
-    xurl_free( psz_scheme );
+    free( psz_scheme );
 
     return b_return_value;
 }
 
-#ifndef HAVE_STRDUP
-static
-char *xurl_strdup( const char *psz_string )
+
+static char *XURL_FindPath( char *psz_url )
 {
-    size_t i_length;
-    char *psz_new_string;
-
-    if( !psz_string ) return NULL;
- 
-    i_length = strlen( psz_string ) + 1;
-    psz_new_string = (char *) xurl_malloc( i_length );
-    if( psz_new_string == NULL ) return NULL;
-
-    memcpy( psz_new_string, psz_string, i_length );
-
-    return psz_new_string;
-}
-#endif
-
-static
-char *XURL_FindPath( char *psz_url )
-{
-    char *psz_return_value = NULL;
-
-    if( XURL_IsAbsolute( psz_url ) == XURL_TRUE )
+    if( XURL_IsAbsolute( psz_url ) )
     {
         char *psz_start_of_hostname = XURL_FindHostname( psz_url );
         if( psz_start_of_hostname != NULL )
-        {
-            char *psz_start_of_path = strchr( psz_start_of_hostname, '/' );
-            psz_return_value = psz_start_of_path;
-        }
+            return strchr( psz_start_of_hostname, '/' );
+        else
+           return NULL;
     }
     else
     {
-        if( XURL_HasAbsolutePath( psz_url ) == XURL_TRUE )
-        {
-            psz_return_value = psz_url;
-        }
+        if( XURL_HasAbsolutePath( psz_url ) == true )
+            return psz_url;
         else
-        {
-            return xurl_strdup (".");
-        }
+            return strdup (".");
     }
-
-    return psz_return_value;
 }
 
 
@@ -368,7 +326,7 @@ char *XURL_GetPath( char *psz_url )
     char *pc_question_mark = NULL;
     char *pc_fragment = NULL;
 
-    psz_path = xurl_strdup( XURL_FindPath( psz_url ) );
+    psz_path = strdup( XURL_FindPath( psz_url ) );
 #ifdef XURL_DEBUG
     fprintf( stderr, "XURL_GetPath: XURL_FindPath returning \"%s\"\n",
              psz_path );
@@ -417,17 +375,15 @@ char *XURL_GetHead( const char *psz_path )
 #endif
     if( pc_last_slash == NULL )
     {
-        psz_path_head = xurl_strdup( psz_path );
+        psz_path_head = strdup( psz_path );
     }
     else
     {
         size_t i_characters_until_last_slash;
 
         i_characters_until_last_slash = pc_last_slash - psz_path;
-        psz_path_head = malloc(
-                ( i_characters_until_last_slash + 1 ) * sizeof(char) );
-        (void) strncpy( psz_path_head, psz_path,
-                i_characters_until_last_slash + 1 );
+        psz_path_head = malloc( i_characters_until_last_slash + 1 );
+        strncpy( psz_path_head, psz_path, i_characters_until_last_slash + 1 );
 
         /* terminate the resulting string with '\0' */
         *(psz_path_head +
@@ -449,7 +405,7 @@ char *XURL_GetWithoutFragment( char *psz_url )
     psz_fragment = XURL_FindFragment( psz_url );
     if( psz_fragment == NULL )
     {
-        psz_return_value = xurl_strdup( psz_url );
+        psz_return_value = strdup( psz_url );
     }
     else
     {
@@ -458,7 +414,7 @@ char *XURL_GetWithoutFragment( char *psz_url )
 
         i_pre_fragment_length = psz_fragment - psz_url;
 
-        psz_without_fragment = xurl_malloc( i_pre_fragment_length + 1 );
+        psz_without_fragment = malloc( i_pre_fragment_length + 1 );
         if( psz_without_fragment == NULL )
         {
             psz_return_value = NULL;
@@ -474,8 +430,7 @@ char *XURL_GetWithoutFragment( char *psz_url )
     return psz_return_value;
 }
 
-static
-char *streallocat( char *psz_string, char *psz_to_append )
+static char *streallocat( char *psz_string, const char *psz_to_append )
 {
     size_t i_new_string_length = strlen( psz_string ) +
         strlen( psz_to_append ) + 1;
