@@ -38,25 +38,6 @@ struct decoder_sys_t
     uint8_t buffer[65536];
 };
 
-typedef struct subpicture_data_t
-{
-    mtime_t i_pts;                                 /* presentation timestamp */
-
-    int   pi_offset[2];                              /* byte offsets to data */
-    void *p_data;
-
-    /* Color information */
-    bool b_palette;
-    uint8_t    pi_alpha[4];
-    uint8_t    pi_yuv[4][3];
-
-    /* Auto crop fullscreen subtitles */
-    bool b_auto_crop;
-    int i_y_top_offset;
-    int i_y_bottom_offset;
-
-} subpicture_data_t;
-
 /*****************************************************************************
  * Amount of bytes we GetChunk() in one go
  *****************************************************************************/
