@@ -2,7 +2,7 @@
  * interface_widgets.hpp : Playlist Widgets
  ****************************************************************************
  * Copyright (C) 2006 the VideoLAN team
- * $Id: 701b913298f680d8803ea52c138c4da77a7b9d22 $
+ * $Id$
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -30,9 +30,10 @@
 # include "config.h"
 #endif
 
-#include <vlc_common.h>
 #include "qt4.hpp"
-#include "dialogs_provider.hpp"
+
+#include "dialogs_provider.hpp" /* Media Info from ArtLabel */
+#include "components/interface_widgets.hpp"
 
 #include <QSplitter>
 #include <QLabel>
@@ -40,38 +41,40 @@
 class PLSelector;
 class PLPanel;
 class QPushButton;
+class CoverArtLabel;
+class ArtLabel;
 
 class PlaylistWidget : public QSplitter
 {
     Q_OBJECT;
 public:
-    PlaylistWidget( intf_thread_t *_p_i, QWidget *parent ) ;
+    PlaylistWidget( intf_thread_t *_p_i );
     virtual ~PlaylistWidget();
 private:
     PLSelector *selector;
     PLPanel *rightPanel;
     QPushButton *addButton;
-    QLabel *art;
-    QString prevArt;
-    QWidget *parent;
+    ArtLabel *art;
 protected:
     intf_thread_t *p_intf;
     virtual void dropEvent( QDropEvent *);
     virtual void dragEnterEvent( QDragEnterEvent * );
     virtual void closeEvent( QCloseEvent * );
 
-private slots:
-    void setArt( QString );
 signals:
     void rootChanged( int );
 };
 
-class ArtLabel : public QLabel
+class ArtLabel : public CoverArtLabel
 {
     Q_OBJECT
-    void mouseDoubleClickEvent( QMouseEvent *event )
+public:
+    ArtLabel( QWidget *parent, intf_thread_t *intf )
+            : CoverArtLabel( parent, intf ) {};
+    virtual void mouseDoubleClickEvent( QMouseEvent *event )
     {
         THEDP->mediaInfoDialog();
+        event->accept();
     }
 };
 
