@@ -2,7 +2,7 @@
  * dialogs_provider.cpp : Dialog Provider
  *****************************************************************************
  * Copyright (C) 2006-2009 the VideoLAN team
- * $Id: 0c4da89b77805ce85b15a00fc0d5b36e6a3c62dd $
+ * $Id: 89cd76ca1e523e0a931057ffebebf0f8e5160d65 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -532,7 +532,7 @@ void DialogsProvider::saveAPlaylist()
         char filter[24];
         char module[12];
     } types[] = {
-        { N_("XSPF playlist (*.xpsf)"), "export-xspf", },
+        { N_("XSPF playlist (*.xspf)"), "export-xspf", },
         { N_("M3U playlist (*.m3u)"), "export-m3u", },
         { N_("HTML playlist (*.html)"), "export-html", },
     };
@@ -569,7 +569,7 @@ void DialogsProvider::streamingDialog( QWidget *parent,
                                        bool b_transcode_only,
                                        QStringList options )
 {
-    char *psz_soutoption;
+    QString soutoption;
 
     /* Stream */
     if( !b_transcode_only )
@@ -577,7 +577,7 @@ void DialogsProvider::streamingDialog( QWidget *parent,
         SoutDialog *s = new SoutDialog( parent, p_intf, mrl );
         if( s->exec() == QDialog::Accepted )
         {
-            psz_soutoption = strdup( qtu( s->getMrl() ) );
+            soutoption = s->getMrl();
             delete s;
         }
         else
@@ -589,7 +589,7 @@ void DialogsProvider::streamingDialog( QWidget *parent,
         ConvertDialog *s = new ConvertDialog( parent, p_intf, mrl );
         if( s->exec() == QDialog::Accepted )
         {
-            psz_soutoption = strdup( qtu( s->getMrl() ) );
+            soutoption = s->getMrl();
             delete s;
         }
         else
@@ -599,9 +599,9 @@ void DialogsProvider::streamingDialog( QWidget *parent,
     }
 
     /* Get SoutMRL */
-    if( !EMPTY_STR( psz_soutoption ) )
+    if( !soutoption.isEmpty() )
     {
-        options += QString( psz_soutoption ).split( " :");
+        options += soutoption.split( " :");
 
         /* Create Input */
         input_item_t *p_input;
@@ -627,7 +627,6 @@ void DialogsProvider::streamingDialog( QWidget *parent,
 
         RecentsMRL::getInstance( p_intf )->addRecent( mrl );
     }
-    free( psz_soutoption );
 }
 
 void DialogsProvider::openAndStreamingDialogs()
