@@ -2,7 +2,7 @@
  * ustring.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: 3d6476dfbc92a8ecd49f613b7063fefb6907af60 $
+ * $Id$
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulière <ipkiss@via.ecp.fr>
@@ -199,8 +199,11 @@ bool UString::operator >=( const UString &rOther ) const
 }
 
 
-void UString::operator =( const UString &rOther )
+UString& UString::operator =( const UString &rOther )
 {
+    if( this == &rOther )
+        return *this;
+
     m_length = rOther.m_length;
     delete[] m_pString;
     m_pString = new uint32_t[size() + 1];
@@ -208,15 +211,20 @@ void UString::operator =( const UString &rOther )
     {
         m_pString[i] = rOther.m_pString[i];
     }
+
+    return *this;
 }
 
 
-void UString::operator +=( const UString &rOther )
+UString& UString::operator +=( const UString &rOther )
 {
+    if( this == &rOther )
+        return *this;
+
     int tempLength = this->length() + rOther.length();
     uint32_t *pTempString = new uint32_t[tempLength + 1];
     // Copy the first string
-    memcpy( pTempString, this->m_pString, 4 * this->size() );
+    memcpy( pTempString, this->m_pString, sizeof(uint32_t) * this->size() );
     // Append the second string
 //     memcpy( pTempString + 4 * size(), rOther.m_pString,
 //             4 * rOther.size() );
@@ -230,6 +238,8 @@ void UString::operator +=( const UString &rOther )
     delete[] m_pString;
     m_pString = pTempString;
     m_length = tempLength;
+
+    return *this;
 }
 
 
@@ -249,19 +259,6 @@ const UString UString::operator +( const char *pString ) const
 }
 
 
-void UString::debug() const
-{
-    char *s = new char[size() + 1];
-    for( uint32_t i = 0; i < size(); i++ )
-    {
-        s[i] = (char)m_pString[i];
-    }
-    s[size()] = '\0';
-    msg_Err( getIntf(), "%s", s );
-    delete[] s;
-}
-
-
 uint32_t UString::find( const UString &str, uint32_t position ) const
 {
     uint32_t pos;
