@@ -2,7 +2,7 @@
  * fileinfo.cpp : WinCE gui plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000-2004 the VideoLAN team
- * $Id: fileinfo.cpp 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id$
  *
  * Authors: Marodon Cedric <cedric_marodon@yahoo.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -19,17 +19,19 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdlib.h>                                      /* malloc(), free() */
-#include <string.h>                                            /* strerror() */
-#include <stdio.h>
-#include <vlc/vlc.h>
-#include <vlc/intf.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_interface.h>
+#include <vlc_input.h>
 
 #include "wince.h"
 
@@ -52,10 +54,10 @@ FileInfo::FileInfo( intf_thread_t *p_intf, CBaseWindow *p_parent,
 
 /***********************************************************************
 
-FUNCTION: 
+FUNCTION:
   CreateTreeView
 
-PURPOSE: 
+PURPOSE:
   Registers the TreeView control class and creates a TreeView.
 
 ***********************************************************************/
@@ -75,7 +77,7 @@ BOOL FileInfo::CreateTreeView(HWND hwnd)
     GetClientRect( hwnd, &rect );
 
     // Assign the window styles for the tree view.
-    dwStyle = WS_VISIBLE | WS_CHILD | TVS_HASLINES | TVS_LINESATROOT | 
+    dwStyle = WS_VISIBLE | WS_CHILD | TVS_HASLINES | TVS_LINESATROOT |
                           TVS_HASBUTTONS;
 
     // Create the tree-view control.
@@ -94,20 +96,20 @@ BOOL FileInfo::CreateTreeView(HWND hwnd)
 
 /***********************************************************************
 
-FUNCTION: 
+FUNCTION:
   UpdateFileInfo
 
-PURPOSE: 
+PURPOSE:
   Update the TreeView with file information.
 
 ***********************************************************************/
 void FileInfo::UpdateFileInfo()
 {
-    TVITEM tvi = {0}; 
-    TVINSERTSTRUCT tvins = {0}; 
-    HTREEITEM hPrev = (HTREEITEM)TVI_FIRST; 
-    HTREEITEM hPrevRootItem = NULL; 
-    HTREEITEM hPrevLev2Item = NULL; 
+    TVITEM tvi = {0};
+    TVINSERTSTRUCT tvins = {0};
+    HTREEITEM hPrev = (HTREEITEM)TVI_FIRST;
+    HTREEITEM hPrevRootItem = NULL;
+    HTREEITEM hPrevLev2Item = NULL;
 
     p_intf->p_sys->p_input = (input_thread_t *)
         vlc_object_find( p_intf, VLC_OBJECT_INPUT, FIND_ANYWHERE );
@@ -116,40 +118,40 @@ void FileInfo::UpdateFileInfo()
 
     if( !p_input ) return;
 
-    tvi.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM; 
+    tvi.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM;
 
-    // Set the text of the item.
-    tvi.pszText = _FROMMB( p_input->input.p_item->psz_name );
+    input_item_t * inp_item = input_GetItem(p_input);
+    tvi.pszText = _FROMMB( input_item_GetName( inp_item ) );
     tvi.cchTextMax = _tcslen( tvi.pszText );
 
     // Save the heading level in the item's application-defined data area
     tvi.lParam = (LPARAM)1;
-    tvins.item = tvi; 
+    tvins.item = tvi;
     //tvins.hInsertAfter = TVI_LAST;
-    tvins.hInsertAfter = hPrev; 
-    tvins.hParent = TVI_ROOT; 
+    tvins.hInsertAfter = hPrev;
+    tvins.hParent = TVI_ROOT;
 
-    // Add the item to the tree-view control. 
+    // Add the item to the tree-view control.
     hPrev = (HTREEITEM)TreeView_InsertItem( hwndTV, &tvins );
 
-    hPrevRootItem = hPrev; 
+    hPrevRootItem = hPrev;
 
-    vlc_mutex_lock( &p_input->input.p_item->lock );
-    for( int i = 0; i < p_input->input.p_item->i_categories; i++ )
+    vlc_mutex_lock( &inp_item->lock );
+    for( int i = 0; i < inp_item->i_categories; i++ )
     {
-        info_category_t *p_cat = p_input->input.p_item->pp_categories[i];
+        info_category_t *p_cat = inp_item->pp_categories[i];
 
-        // Set the text of the item. 
-        tvi.pszText = _FROMMB( p_input->input.p_item->psz_name );
+        // Set the text of the item.
+        tvi.pszText = _FROMMB( inp_item->psz_name );
         tvi.cchTextMax = _tcslen( tvi.pszText );
-        
+ 
         // Save the heading level in the item's application-defined data area
         tvi.lParam = (LPARAM)2; // level 2
-        tvins.item = tvi; 
-        tvins.hInsertAfter = hPrev; 
+        tvins.item = tvi;
+        tvins.hInsertAfter = hPrev;
         tvins.hParent = hPrevRootItem;
 
-        // Add the item to the tree-view control. 
+        // Add the item to the tree-view control.
         hPrev = (HTREEITEM)TreeView_InsertItem( hwndTV, &tvins );
 
         hPrevLev2Item = hPrev;
@@ -158,24 +160,24 @@ void FileInfo::UpdateFileInfo()
         {
             info_t *p_info = p_cat->pp_infos[j];
 
-            // Set the text of the item. 
+            // Set the text of the item.
             string szAnsi = (string)p_info->psz_name;
             szAnsi += ": ";
             szAnsi += p_info->psz_value;
             tvi.pszText = (TCHAR *)_FROMMB( szAnsi.c_str() );
             tvi.cchTextMax = _tcslen( tvi.pszText );
             tvi.lParam = (LPARAM)3; // level 3
-            tvins.item = tvi; 
-            tvins.hInsertAfter = hPrev; 
+            tvins.item = tvi;
+            tvins.hInsertAfter = hPrev;
             tvins.hParent = hPrevLev2Item;
-    
-            // Add the item to the tree-view control. 
+ 
+            // Add the item to the tree-view control.
             hPrev = (HTREEITEM)TreeView_InsertItem( hwndTV, &tvins );
         }
 
         TreeView_Expand( hwndTV, hPrevLev2Item, TVE_EXPANDPARTIAL|TVE_EXPAND );
     }
-    vlc_mutex_unlock( &p_input->input.p_item->lock );
+    vlc_mutex_unlock( &inp_item->lock );
 
     TreeView_Expand( hwndTV, hPrevRootItem, TVE_EXPANDPARTIAL|TVE_EXPAND );
 
@@ -184,12 +186,12 @@ void FileInfo::UpdateFileInfo()
 
 /***********************************************************************
 
-FUNCTION: 
+FUNCTION:
   WndProc
 
-PURPOSE: 
+PURPOSE:
   Processes messages sent to the main window.
-  
+ 
 ***********************************************************************/
 LRESULT FileInfo::WndProc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
 {
@@ -197,7 +199,7 @@ LRESULT FileInfo::WndProc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
 
     switch( msg )
     {
-    case WM_INITDIALOG: 
+    case WM_INITDIALOG:
         shidi.dwMask = SHIDIM_FLAGS;
         shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN |
             SHIDIF_FULLSCREENNOMENUBAR;//SHIDIF_SIZEDLGFULLSCREEN;
@@ -213,7 +215,7 @@ LRESULT FileInfo::WndProc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
         break;
 
     case WM_SETFOCUS:
-        SHSipPreference( hwnd, SIP_DOWN ); 
+        SHSipPreference( hwnd, SIP_DOWN );
         SHFullScreen( hwnd, SHFS_HIDESIPBUTTON );
         break;
 
