@@ -2,7 +2,7 @@
  * demux.c :  Lua playlist demux module
  *****************************************************************************
  * Copyright (C) 2007-2008 the VideoLAN team
- * $Id: b106cf5c2452bf4a1ec87e773454d149b8a1fa6d $
+ * $Id: d7f4445bf0ced0fdaf02b7567cdd43de7a4c8449 $
  *
  * Authors: Antoine Cellerier <dionoea at videolan tod org>
  *
@@ -28,6 +28,8 @@
 # include "config.h"
 #endif
 
+#include <assert.h>
+
 #include <vlc_common.h>
 #include <vlc_demux.h>
 #include <vlc_url.h>
@@ -41,6 +43,7 @@
 
 #include "vlc.h"
 #include "libs.h"
+#include "libs/playlist.h"
 
 
 /*****************************************************************************
@@ -71,10 +74,12 @@ static int vlclua_demux_peek( lua_State *L )
 static int vlclua_demux_read( lua_State *L )
 {
     demux_t *p_demux = (demux_t *)vlclua_get_this( L );
-    uint8_t *p_read;
+    const uint8_t *p_read;
     int n = luaL_checkint( L, 1 );
-    int i_read = stream_Read( p_demux->s, &p_read, n );
+    int i_read = stream_Peek( p_demux->s, &p_read, n );
     lua_pushlstring( L, (const char *)p_read, i_read );
+    int i_seek = stream_Read( p_demux->s, NULL, i_read );
+    assert(i_read==i_seek);
     return 1;
 }
 
@@ -246,7 +251,7 @@ static int Demux( demux_t *p_demux )
     input_thread_t *p_input_thread = (input_thread_t *)
         vlc_object_find( p_demux, VLC_OBJECT_INPUT, FIND_PARENT );
     input_item_t *p_current_input = input_GetItem( p_input_thread );
-    playlist_t *p_playlist = pl_Yield( p_demux );
+    playlist_t *p_playlist = pl_Hold( p_demux );
 
     luaL_register( L, "vlc", p_reg_parse );
 
@@ -256,6 +261,7 @@ static int Demux( demux_t *p_demux )
     {
         msg_Warn( p_demux, "Error while runing script %s, "
                   "function parse() not found", psz_filename );
+        pl_Release( p_demux );
         return VLC_EGENERIC;
     }
 
@@ -264,6 +270,7 @@ static int Demux( demux_t *p_demux )
         msg_Warn( p_demux, "Error while runing script %s, "
                   "function parse(): %s", psz_filename,
                   lua_tostring( L, lua_gettop( L ) ) );
+        pl_Release( p_demux );
         return VLC_EGENERIC;
     }
 
